@@ -29,10 +29,6 @@
 #include <QUrl>
 #include <QtDebug>
 
-#ifdef BLOGS
-#include "gui/unfinished/blogs/BlogsDialog.h"
-#endif 
-
 #include <retroshare/rsplugin.h>
 #include <retroshare/rsconfig.h>
 
@@ -43,9 +39,9 @@
 #include "HomePage.h"
 #include "NetworkDialog.h"
 #include "gui/FileTransfer/SearchDialog.h"
+#include "gui/FileTransfer/SharedFilesDialog.h"
 #include "gui/FileTransfer/TransfersDialog.h"
 #include "MessagesDialog.h"
-#include "SharedFilesDialog.h"
 #include "PluginsPage.h"
 #include "NewsFeed.h"
 #include "ShareManager.h"
@@ -87,6 +83,7 @@
 #include "statusbar/SoundStatus.h"
 #include "statusbar/ToasterDisable.h"
 #include "statusbar/SysTrayStatus.h"
+#include "statusbar/torstatus.h"
 #include <retroshare/rsstatus.h>
 
 #include <retroshare/rsiface.h>
@@ -107,7 +104,7 @@
 #include "gui/statistics/StatisticsWindow.h"
 
 #include "gui/connect/ConnectFriendWizard.h"
-#include "gui/common/RsCollectionFile.h"
+#include "gui/common/RsCollection.h"
 #include "settings/rsettingswin.h"
 #include "settings/rsharesettings.h"
 #include "settings/WebuiPage.h"
@@ -145,6 +142,7 @@
 #define IMAGE_BLOGS             ":/images/kblogger.png"
 #define IMAGE_DHT               ":/images/dht16.png"
 
+/*static*/ bool MainWindow::hiddenmode = false;
 
 /*static*/ MainWindow *MainWindow::_instance = NULL;
 
@@ -186,6 +184,8 @@ MainWindow::MainWindow(QWidget* parent, Qt::WindowFlags flags)
     RsPeerDetails pd;
     if (rsPeers->getPeerDetails(rsPeers->getOwnId(), pd)) {
         nameAndLocation = QString("%1 (%2)").arg(QString::fromUtf8(pd.name.c_str())).arg(QString::fromUtf8(pd.location.c_str()));
+        if(pd.netMode == RS_NETMODE_HIDDEN)
+            hiddenmode = true;
     }
 
     setWindowTitle(tr("RetroShare %1 a secure decentralized communication platform").arg(Rshare::retroshareVersion(true)) + " - " + nameAndLocation);
@@ -200,7 +200,7 @@ MainWindow::MainWindow(QWidget* parent, Qt::WindowFlags flags)
     this->setWindowIcon(QIcon(QString::fromUtf8(":/icons/logo_128.png")));
 
     /* Create all the dialogs of which we only want one instance */
-    _bandwidthGraph = new BandwidthGraph();
+    _bandwidthGraph = NULL ;
 
     #ifdef UNFINISHED
     applicationWindow = new ApplicationWindow();
@@ -211,8 +211,9 @@ MainWindow::MainWindow(QWidget* parent, Qt::WindowFlags flags)
     connect(ui->listWidget, SIGNAL(currentRowChanged(int)), this, SLOT(setNewPage(int)));
     connect(ui->stackPages, SIGNAL(currentChanged(int)), this, SLOT(setNewPage(int)));
 
-    //ui->stackPages->setCurrentIndex(Settings->getLastPageInMainWindow());
-    setNewPage(Settings->getLastPageInMainWindow());
+	int lastpageindex = Settings->getLastPageInMainWindow();
+	if(lastpageindex < ui->stackPages->count())	//Do not crash when a page was removed after last run
+		setNewPage(lastpageindex);
 
     ui->splitter->setStretchFactor(0, 0);
     ui->splitter->setStretchFactor(1, 1);
@@ -244,13 +245,36 @@ MainWindow::MainWindow(QWidget* parent, Qt::WindowFlags flags)
     peerstatus->setVisible(Settings->valueFromGroup("StatusBar", "ShowPeer", QVariant(true)).toBool());
     statusBar()->addWidget(peerstatus);
 
-    natstatus = new NATStatus();
-    natstatus->setVisible(Settings->valueFromGroup("StatusBar", "ShowNAT", QVariant(true)).toBool());
-    statusBar()->addWidget(natstatus);
-    
-    dhtstatus = new DHTStatus();
-    dhtstatus->setVisible(Settings->valueFromGroup("StatusBar", "ShowDHT", QVariant(true)).toBool());
-    statusBar()->addWidget(dhtstatus);
+	if(hiddenmode)
+	{
+#ifdef RETROTOR
+		torstatus = new TorStatus();
+		torstatus->setVisible(Settings->valueFromGroup("StatusBar", "ShowTor", QVariant(true)).toBool());
+		statusBar()->addWidget(torstatus);
+		torstatus->getTorStatus();
+#else
+		torstatus = NULL ;
+#endif
+
+		natstatus = NULL ;
+		dhtstatus = NULL ;
+	}
+	else
+	{
+		torstatus = NULL ;
+
+		natstatus = new NATStatus();
+		if(hiddenmode) natstatus->setVisible(false);
+		else natstatus->setVisible(Settings->valueFromGroup("StatusBar", "ShowNAT", QVariant(true)).toBool());
+		statusBar()->addWidget(natstatus);
+		natstatus->getNATStatus();
+
+		dhtstatus = new DHTStatus();
+		if(hiddenmode) dhtstatus->setVisible(false);
+		else dhtstatus->setVisible(Settings->valueFromGroup("StatusBar", "ShowDHT", QVariant(true)).toBool());
+		statusBar()->addWidget(dhtstatus);
+		dhtstatus->getDHTStatus();
+	}
 
     hashingstatus = new HashingStatus();
     hashingstatus->setVisible(Settings->valueFromGroup("StatusBar", "ShowHashing", QVariant(true)).toBool());
@@ -379,10 +403,6 @@ void MainWindow::initStackedPage()
   addPage(wikiDialog = new WikiDialog(ui->stackPages), grp, &notify);
 #endif
 
-#ifdef BLOGS
-  addPage(blogsFeed = new BlogsDialog(ui->stackPages), grp, NULL);
-#endif
-
  std::cerr << "Looking for interfaces in existing plugins:" << std::endl;
  for(int i = 0;i<rsPlugins->nbPlugins();++i)
  {
@@ -567,7 +587,7 @@ void MainWindow::createTrayIcon()
 #ifdef ENABLE_WEBUI
     trayMenu->addAction(QIcon(":/images/emblem-web.png"), tr("Show web interface"), this, SLOT(showWebinterface()));
 #endif // ENABLE_WEBUI
-    trayMenu->addAction(QIcon(IMAGE_BWGRAPH), tr("Bandwidth Graph"), _bandwidthGraph, SLOT(showWindow()));
+    trayMenu->addAction(QIcon(IMAGE_BWGRAPH), tr("Bandwidth Graph"), this, SLOT(showBandwidthGraph()));
     trayMenu->addAction(QIcon(IMAGE_DHT), tr("Statistics"), this, SLOT(showStatisticsWindow()));
 
 
@@ -593,6 +613,14 @@ void MainWindow::createTrayIcon()
     trayIcon->show();
 }
 
+void MainWindow::showBandwidthGraph()
+{
+	if(_bandwidthGraph == NULL)
+		_bandwidthGraph = new BandwidthGraph();
+
+	_bandwidthGraph->showWindow();
+}
+
 void MainWindow::createNotifyIcons()
 {
     /* create notify icons */
@@ -702,11 +730,17 @@ void MainWindow::updateStatus()
     if (ratesstatus)
         ratesstatus->getRatesStatus(downKb, upKb);
 
+	if(torstatus)
+		torstatus->getTorStatus();
+
+    if(!hiddenmode)
+    {
     if (natstatus)
         natstatus->getNATStatus();
         
     if (dhtstatus)
         dhtstatus->getDHTStatus();
+    }
 
     if (discstatus) {
         discstatus->update();
@@ -915,11 +949,6 @@ void SetForegroundWindowInternal(HWND hWnd)
 		 case Forums:
                          _instance->ui->stackPages->setCurrentPage( _instance->gxsforumDialog );
                          return true ;
-#ifdef BLOGS
-		 case Blogs:
-			 Page = _instance->blogsFeed;
-			 return true ;
-#endif
 		case Posted:
 			_instance->ui->stackPages->setCurrentPage( _instance->postedDialog );
 			return true ;
@@ -960,11 +989,6 @@ void SetForegroundWindowInternal(HWND hWnd)
    if (page == _instance->messagesDialog) {
        return Messages;
    }
-#ifdef RS_USE_LINKS
-   if (page == _instance->linksDialog) {
-       return Links;
-   }
-#endif
 #if 0
    if (page == _instance->channelFeed) {
        return Channels;
@@ -973,11 +997,6 @@ void SetForegroundWindowInternal(HWND hWnd)
        return Forums;
    }
 #endif
-#ifdef BLOGS
-   if (page == _instance->blogsFeed) {
-       return Blogs;
-   }
-#endif
 
    return Network;
 }
@@ -1009,20 +1028,12 @@ void SetForegroundWindowInternal(HWND hWnd)
 			return _instance->transfersDialog->searchDialog;
 		case Messages:
 			return _instance->messagesDialog;
-#ifdef RS_USE_LINKS
-		case Links:
-			return _instance->linksDialog;
-#endif
 		case Channels:
 			return _instance->gxschannelDialog;
 		case Forums:
 			return _instance->gxsforumDialog;
 		case Posted:
 			return _instance->postedDialog;
-#ifdef BLOGS
-		case Blogs:
-			return _instance->blogsFeed;
-#endif
 	}
 
    return NULL;
@@ -1039,12 +1050,12 @@ void MainWindow::addFriend()
 }
 
 /** New RSCollection ShortCut */
-void MainWindow::newRsCollection()
-{
-    std::vector <DirDetails> dirVec;
-
-    RsCollectionFile(dirVec).openNewColl(this);
-}
+// void MainWindow::newRsCollection()
+// {
+//     std::vector <DirDetails> dirVec;
+//
+//     RsCollection(dirVec).openNewColl(this);
+// }
 
 /** Shows Share Manager */
 void MainWindow::openShareManager()
@@ -1471,8 +1482,8 @@ void MainWindow::openRsCollection(const QString &filename)
 {
 	QFileInfo qinfo(filename);
 	if (qinfo.exists()) {
-		if (qinfo.absoluteFilePath().endsWith(RsCollectionFile::ExtensionString)) {
-			RsCollectionFile collection;
+		if (qinfo.absoluteFilePath().endsWith(RsCollection::ExtensionString)) {
+			RsCollection collection;
 			collection.openColl(qinfo.absoluteFilePath());
 		}
 	}
@@ -1501,7 +1512,7 @@ void MainWindow::processLastArgs()
 			opModeStatus->setCurrentIndex(RS_OPMODE_GAMING - 1);
 		} else if (opmode == "minimal") {
 			opModeStatus->setCurrentIndex(RS_OPMODE_MINIMAL - 1);
-		} else {
+		} else if (opmode != "") {
 			opModeStatus->setCurrentIndex(RS_OPMODE_FULL - 1);
 		}
 		opModeStatus->setOpMode();
@@ -1599,10 +1610,17 @@ void MainWindow::setCompactStatusMode(bool compact)
 	//statusComboBox: TODO Show only icon
 	peerstatus->setCompactMode(compact);
 	updateFriends();
+    if(!hiddenmode)
+    {
 	natstatus->setCompactMode(compact);
 	natstatus->getNATStatus();
 	dhtstatus->setCompactMode(compact);
 	dhtstatus->getDHTStatus();
+    }
+
+	if(torstatus)
+		torstatus->setCompactMode(compact) ;
+
 	hashingstatus->setCompactMode(compact);
 	ratesstatus->setCompactMode(compact);
 	//opModeStatus: TODO Show only ???
