@@ -35,6 +35,8 @@
 #include "HandleRichText.h"
 #include "gui/RetroShareLink.h"
 #include "util/ObjectPainter.h"
+#include "util/imageutil.h"
+#include "util/rstime.h"
 
 #include <iostream>
 
@@ -237,6 +239,8 @@ bool RsHtml::canReplaceAnchor(QDomDocument &/*doc*/, QDomElement &/*element*/, c
 	case RetroShareLink::TYPE_PUBLIC_MSG:
 	case RetroShareLink::TYPE_POSTED:
 	case RetroShareLink::TYPE_IDENTITY:
+	case RetroShareLink::TYPE_FILE_TREE:
+	case RetroShareLink::TYPE_CHAT_ROOM:
 		// not yet implemented
 		break;
 
@@ -267,6 +271,8 @@ void RsHtml::anchorStylesheetForImg(QDomDocument &/*doc*/, QDomElement &/*elemen
 	case RetroShareLink::TYPE_PUBLIC_MSG:
 	case RetroShareLink::TYPE_POSTED:
 	case RetroShareLink::TYPE_IDENTITY:
+	case RetroShareLink::TYPE_FILE_TREE:
+	case RetroShareLink::TYPE_CHAT_ROOM:
 		// not yet implemented
 		break;
 
@@ -384,6 +390,20 @@ void RsHtml::embedHtml(QTextDocument *textDocument, QDomDocument& doc, QDomEleme
 								replaceAnchorWithImg(doc, element, textDocument, link);
 							}
 						}
+						else
+						{
+							QUrl url(element.attribute("href"));
+							if(url.isValid())
+							{
+								QString title = url.host();
+								if (!title.isEmpty()) {
+									element.setAttribute("title", title);
+								}
+								if (textDocument && (flag & RSHTML_FORMATTEXT_REPLACE_LINKS)) {
+									replaceAnchorWithImg(doc, element, textDocument, url);
+								}
+							}
+						}
 					} else {
 						if (textDocument && (flag & RSHTML_FORMATTEXT_REPLACE_LINKS)) {
 							RetroShareLink link(element.attribute("href"));
@@ -439,6 +459,20 @@ void RsHtml::embedHtml(QTextDocument *textDocument, QDomDocument& doc, QDomEleme
 										replaceAnchorWithImg(doc, insertedTag, textDocument, link);
 									}
 								}
+								else
+								{
+									QUrl url(myRE.cap(0));
+									if(url.isValid())
+									{
+										QString title = url.host();
+										if (!title.isEmpty()) {
+											insertedTag.setAttribute("title", title);
+										}
+										if (textDocument && (flag & RSHTML_FORMATTEXT_REPLACE_LINKS)) {
+											replaceAnchorWithImg(doc, insertedTag, textDocument, url);
+										}
+									}
+								}
 							}
 							break;
 					case Img:
@@ -1105,7 +1139,7 @@ QString RsHtml::toHtml(QString text, bool realHtml)
 }
 
 /** Loads image and converts image to embedded image HTML fragment **/
-bool RsHtml::makeEmbeddedImage(const QString &fileName, QString &embeddedImage, const int maxPixels)
+bool RsHtml::makeEmbeddedImage(const QString &fileName, QString &embeddedImage, const int maxPixels, const int maxBytes)
 {
 	QImage image;
 
@@ -1113,54 +1147,15 @@ bool RsHtml::makeEmbeddedImage(const QString &fileName, QString &embeddedImage,
 		fprintf (stderr, "RsHtml::makeEmbeddedImage() - image \"%s\" can't be load\n", fileName.toLatin1().constData());
 		return false;
 	}
-	return RsHtml::makeEmbeddedImage(image, embeddedImage, maxPixels);
+	return RsHtml::makeEmbeddedImage(image, embeddedImage, maxPixels, maxBytes);
 }
 
 /** Converts image to embedded image HTML fragment **/
-bool RsHtml::makeEmbeddedImage(const QImage &originalImage, QString &embeddedImage, const int maxPixels)
+bool RsHtml::makeEmbeddedImage(const QImage &originalImage, QString &embeddedImage, const int maxPixels, const int maxBytes)
 {
-	QByteArray bytearray;
-	QBuffer buffer(&bytearray);
-	QImage resizedImage;
-	const QImage *image = &originalImage;
-
-	if (maxPixels > 0) {
-		QSize imgSize = originalImage.size();
-		if ((imgSize.height() * imgSize.width()) > maxPixels) {
-			// image is too large - resize keeping aspect ratio
-			QSize newSize;
-			newSize.setWidth(int(qSqrt((maxPixels * imgSize.width()) / imgSize.height())));
-			newSize.setHeight(int((imgSize.height() * newSize.width()) / imgSize.width()));
-
-			// ask user
-			QMessageBox msgBox;
-			msgBox.setText(QString(QApplication::translate("RsHtml", "Image is oversized for transmission.\nReducing image to %1x%2 pixels?")).arg(newSize.width()).arg(newSize.height()));
-			msgBox.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
-			msgBox.setDefaultButton(QMessageBox::Ok);
-			if (msgBox.exec() != QMessageBox::Ok) {
-				return false;
-			}
-			resizedImage = originalImage.scaled(newSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
-			image = &resizedImage;
-		}
-	}
-
-	if (buffer.open(QIODevice::WriteOnly)) {
-		if (image->save(&buffer, "PNG")) {
-			QByteArray encodedByteArray = bytearray.toBase64();
-
-			embeddedImage = "<img src=\"data:image/png;base64,";
-			embeddedImage.append(encodedByteArray);
-			embeddedImage.append("\">");
-		} else {
-            //fprintf (stderr, "RsHtml::makeEmbeddedImage() - image can't be saved to buffer\n");
-			return false;
-		}
-	} else {
-		fprintf (stderr, "RsHtml::makeEmbeddedImage() - buffer can't be opened\n");
-		return false;
-	}
-	return true;
+	rstime::RsScopeTimer s("Embed image");
+	QImage opt;
+	return ImageUtil::optimizeSize(embeddedImage, originalImage, opt, maxPixels, maxBytes);
 }
 
 QString RsHtml::plainText(const QString &text)
